
//----------------------------------------------------------------------------
//  File:   AmiSocket.cpp
//
//  Desc:   Windows Message Based UDP class for AMINet and generic UDP
//
//  c.2003  Alcorn McBride - Scott Harkless
//----------------------------------------------------------------------------

#include "stdafx.h"
#include "AmiSocket.h"


//----------------------------------------------------------------------------
// Constructor 
//----------------------------------------------------------------------------
CAmiSocket::CAmiSocket()
{
}

//----------------------------------------------------------------------------
// Destructor 
//----------------------------------------------------------------------------
CAmiSocket::~CAmiSocket()
{
}


//----------------------------------------------------------------------------
// Initialize and Open UDP Socket
//----------------------------------------------------------------------------

// hWnd			= The handle to your window
// DestAddress  = The default destination address of your messages - Defaults to value of AMINET_ADDRESS
// DestPort		= The default destination port - Use AMINET_PORT
// DecodeAMINet = Set to TRUE to decode incoming AMINet Messages, FALSE to just pass on all data - Defaults to FALSE

bool CAmiSocket::InitAndOpenUDPSocket(HWND hWnd, UINT Port, bool DecodeAMINet)
{
	bool bOptVal = TRUE;
	
	if(hWnd!=NULL) 
	{
		m_hWnd = hWnd; 
		m_nPort = Port;
		m_bDecodeAMINet = DecodeAMINet;

		SetSockOpt(SO_BROADCAST, (char far*)&bOptVal, sizeof(BOOL), SOL_SOCKET );
		Create(m_nPort, SOCK_DGRAM, FD_READ);

		return true;
	}

	return false;
}


//----------------------------------------------------------------------------
// Send an Ordinary UDP Message
//----------------------------------------------------------------------------
bool CAmiSocket::SendUDPMessage(CString UserMessage, CString DestAddress)
{
	UserMessage.AppendChar( 0x0D );
	int nResult = SendTo(UserMessage, UserMessage.GetLength(), m_nPort, DestAddress); 

	if(nResult == SOCKET_ERROR)
		return FALSE;
	else
		return TRUE;
}


//----------------------------------------------------------------------------
// Send an AMINet Wrapped Message
//----------------------------------------------------------------------------
bool CAmiSocket::SendAMINetMessage(CString UserMessage, CString DestAddress)
{

	AMINetBuf.buf[0] = (char) 0xF1;
	AMINetBuf.buf[1] = (char) 0x01;
	AMINetBuf.buf[2] = (char) 0x04;
	AMINetBuf.length = 3;
	
	for(int x=0; x<UserMessage.GetLength(); x++)
		AMINetBuf.buf[AMINetBuf.length + x] = UserMessage[x];
	
	AMINetBuf.length += UserMessage.GetLength() ;

	AMINetBuf.buf[AMINetBuf.length] = (char) 0x0D;
	AMINetBuf.length += 1;

	CalculateAMINetChecksum(&AMINetBuf);

	AMINetBuf.buf[AMINetBuf.length] = (char) 0xF2;
	AMINetBuf.length += 1;

	int nResult = SendTo(AMINetBuf.buf, AMINetBuf.length, m_nPort, DestAddress);

	if(nResult == SOCKET_ERROR)
		return FALSE;
	else
		return TRUE;
}




/************************************
*                                   *
*     Private Member Functions      *
*                                   *
************************************/

//----------------------------------------------------------------------------
// UDP Recieve Event 
//----------------------------------------------------------------------------
void CAmiSocket::OnReceive(int nErrorCode)
{
	// Initialize RX Buffer Structure before getting data
	RxUDPBuffer.buf[0] = NULL;
	RxUDPBuffer.length = -1;
	RxUDPBuffer.AMINet = FALSE;

	// Receive the message
	RxUDPBuffer.length = ReceiveFrom(&RxUDPBuffer.buf, UDP_BUFFER_SIZE, RxUDPBuffer.Address, RxUDPBuffer.Port);

	// did we get anything?
	if (RxUDPBuffer.length > 0)
	{
		// Truncate the end of the message
		RxUDPBuffer.buf[RxUDPBuffer.length] = NULL;
		
		// Post a windows message containing the recieved data
		if((m_hWnd!=NULL))
		{
			// If this object is setup to Receive AMINet Messages, process and unwrap the message before posting
			if( m_bDecodeAMINet && (RxUDPBuffer.buf[0] == (char) 0xf1) && (RxUDPBuffer.buf[RxUDPBuffer.length-1] == (char) 0xf2))
			{
				// Strip off AMINet Layer - If it is an invalid AMINet Message, abort
				if(!ProcessAMINet(&RxUDPBuffer))
					return;
				else
					RxUDPBuffer.AMINet = TRUE;  // Set Flag Inform the user that this is an unwrapped AMI-Net Message
			}
			
			// Post a Message containing the recieved UDP message
			PostMessage(m_hWnd, WM_UDP_RECEIVE, NULL, (LPARAM)&RxUDPBuffer);
		}
	}
}




//----------------------------------------------------------------------------
// Process AMINet Packet - Verify Message and Strip off AMINet Wrapper
//----------------------------------------------------------------------------
bool CAmiSocket::ProcessAMINet(UDPBuffer * AMIMsg)
{
	int nCmdLen;
	unsigned long nChecksum = 0;
	unsigned long nCalculatedChecksum = 0; 
	int i, j;
	
	nCmdLen = AMIMsg->length;
	
	//**** Check for Errors in Message - AMINet ****//
	
	char test = AMIMsg->buf[nCmdLen-1];

	// Too short to be a valid command
	if (nCmdLen < 8)								
		return false; 

	// STX byte missing or out of place
	else if (AMIMsg->buf[0] != (char)0xF1)			
		return false;  

	// ETX byte missing or out of place
	else if (AMIMsg->buf[nCmdLen-1] != (char)0xF2)
		return false; 

	// At this point, Message may be Valid
	else
	{
		//Calculate and verify the checksum
		for (i=1; i<nCmdLen-1; i++) //nCmdLen-1 is the last byte of the checksum (first and last of an 8 bit checksum)
		{
			if ( (AMIMsg->buf[i] == (char)0xFF) )
			{
				//reached 16 bit expanded checksum
				for (j=0;j<2;j++)
				{
					i++;
					nChecksum *= 256;
					nChecksum += (AMIMsg->buf[i] & 0xFF);
				}
				break;
			}
			
			else if ( (AMIMsg->buf[i] == (char)0xFE) )
			{
				//reached 24 bit expanded checksum
				for (j=0; j<3; j++)
				{
					i++;
					nChecksum *= 256;
					nChecksum += (AMIMsg->buf[i] & 0xFF);
				}
				break;
			}
			
			else if (i == nCmdLen-2)
			{
				nChecksum = (long)(AMIMsg->buf[i] & 0xFF); //AND w/ 0xFF to clear top unused 24 bits
				break;
			}
			else
				nCalculatedChecksum += (long)AMIMsg->buf[i];
		}
		
		// If checksum is bad, ignore message
		if (nCalculatedChecksum != nChecksum)
			return false; 	
	}


	// If Program gets here, the message has passed AMINet layer

	// AMINet Data no longer necessary, strip off
	CString Command = AMIMsg->buf;
	Command.Delete(0,AMINET_HEADER_LEN);

	int nAtIndex = Command.Find(0x0D);

	// If a 0x0D is not found, there is no valid message - Chop off everything after 0x0d
	if(nAtIndex != -1)
    	Command.Delete(nAtIndex+1,Command.GetLength()-nAtIndex);
	else
		return false;		// no valid message within

	// Re-Copy Unwrapped Data to the UDP Receive Buffer and terminate
	strncpy(AMIMsg->buf, Command, Command.GetLength());
	AMIMsg->buf[Command.GetLength()] = NULL;

	return true;
}



//----------------------------------------------------------------------------
// Calculate AMINet Checksum, then expand and append to string
//----------------------------------------------------------------------------
void CAmiSocket::CalculateAMINetChecksum (UDPBuffer * Message)
{
	unsigned int nChecksum = 0;
	int i;

	// Calculate Checksum
	for (i = 1; i < Message->length; i++)
		nChecksum += Message->buf[i];

	// Translate Checksum Value into a String
	if (nChecksum >= 65536)
	{
		Message->buf[Message->length]	= (char) 0xFE;
		Message->buf[Message->length+1] = (char)((nChecksum & 0xFF0000) >> 16);
		Message->buf[Message->length+2] = (char)((nChecksum & 0xFF00) >> 8);
		Message->buf[Message->length+3] = (char) (nChecksum & 0xFF);
		Message->length += 4;
	}
	else if (nChecksum >= 250)
	{
		Message->buf[Message->length]	= (char) 0xFF;
		Message->buf[Message->length+1]	= (char) ((nChecksum & 0xFF00) >> 8);
		Message->buf[Message->length+2]	= (char) (nChecksum & 0xFF);
		Message->length += 3;
	}
	else
	{
		Message->buf[Message->length] = (char) (nChecksum);
		Message->length += 1;
	}
}