
//----------------------------------------------------------------------------
//  File:   AmiSocket.h
//
//  Desc:   Event-Based UDP class for AMINet
//
//  c.2003  Alcorn McBride - Scott Harkless
//----------------------------------------------------------------------------

#pragma once

// AMINet Related Definitions
#define AMINET_PORT			2639
#define UDP_BUFFER_SIZE		1024
#define AMINET_HEADER_LEN	3

//Your application cannot duplicate this number, start 
//at WM_APP + 3 for other Windows Messages you create
#define WM_UDP_RECEIVE		WM_APP + 2

// UDPBuffer Structure
typedef struct UDPBuffer
{
	int			length;					// Tx and Rx - Length of data in buf
	char		buf[UDP_BUFFER_SIZE];	// Tx and Rx - Data to be sent or data that was received
	CString		Address;				// Rx Only - Address that the command came from
	UINT		Port;					// Rx Only - Port that the message came from
	bool		AMINet;					// Rx Only - Set if the message is an AMINet Message
}UDPBuffer;


//----------------------------------------------------------------------------
// AmiSocket Class - Inherited from the CAsyncSocket Class 
//----------------------------------------------------------------------------
class CAmiSocket : public CAsyncSocket
{

public:
	CAmiSocket();
	virtual ~CAmiSocket();

	// Interface Functions
	bool		InitAndOpenUDPSocket(HWND hWnd, UINT Port, bool DecodeAMINet = FALSE);
	bool		SendUDPMessage(CString Message, CString DestAddress);
	bool		SendAMINetMessage(CString UserMessage, CString DestAddress);

	//variables
	HWND		m_hWnd;
	UINT		m_nPort;
	UDPBuffer	RxUDPBuffer;

	
private:
	void		OnReceive(int nErrorCode);
	bool		ProcessAMINet(UDPBuffer * AMIMsg);
	void		CalculateAMINetChecksum (UDPBuffer * Message);
	bool		m_bDecodeAMINet;
	UDPBuffer	AMINetBuf;

};

